%Copyright (C) 2018 Peter Munro
%
%This program is free software: you can redistribute it and/or modify
%it under the terms of the GNU Lesser Public License as published by
%the Free Software Foundation, either version 3 of the License, or
%(at your option) any later version.
%
%This program is distributed in the hope that it will be useful,
%but WITHOUT ANY WARRANTY; without even the implied warranty of
%MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%GNU Lesser Public License for more details.
%    
%You should have received a copy of the GNU Lesser Public License
%along with this program.  If not, see <http://www.gnu.org/licenses/>.

%Calculate aberrations as a function of incident angle

%refractive index profile
nvec = [1. 1.54 1.33];
hvec = [-170e-6 0];

%beam parameters
lambda = 488e-9;
NA = 0.9;

%calculate angles and weights for integrating the aberration on the
%interval (0,1)
[theta_vec,wtheta_vec] = gauss_legendre(0,asin(NA/nvec(1)),1001);

%Transmission coefficients for various configurations
%Case 0: Propagation in free space
T0 = zeros(size(theta_vec));
%Case 1: Propagation through stratified medium as specified by nvec
%and hvec above
T1 = zeros(size(theta_vec));
%Case 2: Propagation through stratified medium as specified by nvec
%and hvec above, plus an additional 50 microns of water
T2 = zeros(size(theta_vec));

addpath('/home/pmunro/code/focussed_multilayer');
for i=1:numel(theta_vec)
    [ST0] = multi_layer( hvec, [1 1 1]*nvec(1), lambda, theta_vec(i), 'TE');
    [ST1] = multi_layer( hvec, nvec, lambda, theta_vec(i), 'TE');
    [ST2] = multi_layer( [hvec 50e-6], [nvec 1.33], lambda, theta_vec(i), 'TE');
    
    %Transmission coefficients for the above scenarios
    T0(i) = 1/ST0(1,1);
    T1(i) = 1/ST1(1,1);
    T2(i) = 1/ST2(1,1);
end
rmpath('/home/pmunro/code/focussed_multilayer');

%Calulcate the total phase introduced by the stratified medium
%relative to fucussing in free space
W = unwrap(angle(T1./T0));

%%Calulcate the total phase introduced by the stratified medium
%plus 50 microns of water, relative to fucussing in free space
W2 = unwrap(angle(T2./T0));

%The normalised radial pupil parameter
rho = sin(theta_vec)/NA;
save aberation_data_04 W rho;

%Indices of the fringe polynomials to be considered
fringe_vec = [1 4 9 16 25 36];
A = zeros(numel(fringe_vec),numel(fringe_vec));
C = zeros(1,numel(fringe_vec));

for i=1:numel(fringe_vec)
    for j=1:numel(fringe_vec)
	%Calculate the inner product of each fringe polynomial,
        %including with itself
	A(i,j) = sum( zernike_fringe_polyval(rho,fringe_vec(i)).*zernike_fringe_polyval(rho,fringe_vec(j)).*rho.*wtheta_vec.*cos(theta_vec)./(NA/nvec(1)));
    end
    %Decompose the aberration onto each fringe polynomial
    C(i) = sum( zernike_fringe_polyval(rho,fringe_vec(i)).*W.*rho.*wtheta_vec.*cos(theta_vec)./(NA/nvec(1)))/A(i,i);
end

%Evaluate the aberration that results if we use a specified number
%of Zernike fringe polynomials
%PSI is the total aberration
PSI = zeros(size(theta_vec));
%Zn is the aberration due to each order of polynomial
Zn = zeros(numel(fringe_vec),numel(theta_vec));
for i=1:numel(fringe_vec)
    PSI = PSI + C(i)*zernike_fringe_polyval(rho,fringe_vec(i));
    Zn(i,:) = C(i)*zernike_fringe_polyval(rho,fringe_vec(i));
end

%Plot aberrations
figure(1);clf;
hp = [0];
subplot(1,3,2);
hp(1) = plot(rho,W);
hold on;
axis([0 1 -10 600]);
set(hp,'LineWidth',2);

ht = text(0,0,'b)');
set(ht,'Units','normalized');
set(ht,'FontSize',14);
set(ht,'Position',[.05 .95 0]);

set(gca,'FontSize',12);
grid on;hl = legend({'$W_T$'},'Location','southeast');
set(hl,'Interpreter','latex');


subplot(1,3,1);
hp = plot(rho,Zn);
set(hp,'LineWidth',2);

set(gca,'FontSize',12);
grid on;
hx = xlabel('$\rho$');
set(hx,'Interpreter','latex');
hy = ylabel('$W$ (rad)');
set(hy,'Interpreter','latex');
%fringe_vec = [1 4 9 16 25 36];
hl = legend({'$Z_1$','$Z_4$','$Z_9$','$Z_{16}$','$Z_{25}$','$Z_{36}$'},'Location','southeast');
set(hl,'Interpreter','latex');
ht = text(0,0,'a)');
set(ht,'Units','normalized');
set(ht,'FontSize',14);
set(ht,'Position',[.05 .95 0]);

hp = [0 0 0 0];
subplot(1,3,3);
hp(1) = plot(rho,W-PSI);
hold on;
hp(2) = plot(rho,W-sum(Zn(1:5,:)));
hp(3) = plot(rho,W-sum(Zn(1:4,:)));
hp(4) = plot(rho,W-sum(Zn(1:3,:)));
set(hp,'LineWidth',2);

ht = text(0,0,'c)');
set(ht,'Units','normalized');
set(ht,'FontSize',14);
set(ht,'Position',[.05 .95 0]);

set(gca,'FontSize',12);
grid on;
hl = legend({'$W_T-\sum_{i=1}^{6}Z_{i^2}$','$W_T-\sum_{i=1}^{5}Z_{i^2}$','$W_T-\sum_{i=1}^{4}Z_{i^2}$','$W_T-\sum_{i=1}^{3}Z_{i^2}$'},'Location','northeast');
set(hl,'Interpreter','latex');
